==> pytagsfs_0.8.0.orig.tar.gz: pytagsfs-0.8.0/modules/pytagsfs/sourcetreemon/gamin_.py <==
# Copyright (c) 2008 Forest Bond.
# This file is part of the pytagsfs software package.
#
# pytagsfs is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# A copy of the license has been included in the COPYING file.

import os

from threading import RLock

from pytagsfs.util import nonatomic, attr_named

from pytagsfs.sourcetreemon import SourceTreeMonitor
from pytagsfs.sourcetreemon.deferred import DeferredSourceTreeMonitor

from pytagsfs.debug import (
  exceptionally_logged,
  verbosely_logged,
  log_critical,
)

from pytagsfs.exceptions import MissingDependency

################################################################################

class GaminSourceTreeMonitor(SourceTreeMonitor):
    lock = None
    watch_monitor = None

    def __init__(self):
        try:
            import gamin
        except ImportError:
            raise MissingDependency('gamin')

        self.gamin = gamin

        self.event_processors = {
          gamin.GAMChanged: 'process_changed',
          gamin.GAMDeleted: 'process_deleted',
          gamin.GAMStartExecuting: 'process_ignored',
          gamin.GAMStopExecuting: 'process_ignored',
          gamin.GAMCreated: 'process_created',
          gamin.GAMMoved: 'process_deleted',
          gamin.GAMAcknowledge: 'process_ignored',
          gamin.GAMExists: 'process_unexpected',
          gamin.GAMEndExist: 'process_unexpected',
        }

        self.lock = RLock()

################################################################################

    # SourceTreeMonitor API

    @exceptionally_logged
    @verbosely_logged
    def start(self, debug = False):
        self.watch_monitor = self.gamin.WatchMonitor()
        self.watch_monitor.no_exists()

    @exceptionally_logged
    @verbosely_logged
    @nonatomic(attr_named('lock'))
    def stop(self):
        for real_path in self.watch_monitor.objects:
            self.watch_monitor.stop_watch(real_path)

    def supports_threads(self):
        return True

    def supports_writes(self):
        return True

    def can_handle_fork(self):
        return True

    @exceptionally_logged
    @verbosely_logged
    def add_source_dir(self, real_path):
        self.watch_dir(real_path)

    @exceptionally_logged
    @verbosely_logged
    def remove_source_dir(self, real_path):
        self.unwatch_dir(real_path)

    @exceptionally_logged
    @verbosely_logged
    def add_source_file(self, real_path):
        pass

    @exceptionally_logged
    @verbosely_logged
    def remove_source_file(self, real_path):
        pass

    @exceptionally_logged
    @verbosely_logged
    def fileno(self):
        return self.watch_monitor.get_fd()

    @exceptionally_logged
    @verbosely_logged
    @nonatomic(attr_named('lock'))
    def process_events(self):
        return self.watch_monitor.handle_events()

################################################################################

    @exceptionally_logged
    @verbosely_logged
    @nonatomic(attr_named('lock'))
    def watch_dir(self, real_path):
        if real_path in self.watch_monitor.objects:
            # already watching
            return

        try:
            self.watch_monitor.watch_directory(
              real_path,
              self.event_callback,
              real_path,
            )
        except self.gamin.GaminException:
            pass

    @exceptionally_logged
    @verbosely_logged
    @nonatomic(attr_named('lock'))
    def unwatch_dir(self, real_path):
        if real_path not in self.watch_monitor.objects:
            return

        try:
            self.watch_monitor.stop_watch(real_path)
        except self.gamin.GaminException:
            pass

    @exceptionally_logged
    @verbosely_logged
    def process_changed(self, real_path):
        if real_path.endswith(os.path.sep):
            log_critical(
              u'GaminSourceTreeMonitor.process_changed: ignoring %s' %
              real_path
            )
            return
        return self.update_cb(real_path)

    @exceptionally_logged
    @verbosely_logged
    def process_deleted(self, real_path):
        if real_path.endswith(os.path.sep):
            log_critical(
              u'GaminSourceTreeMonitor.process_deleted: ignoring %s' %
              real_path
            )
            return
        return self.remove_cb(real_path)

    @exceptionally_logged
    @verbosely_logged
    def process_ignored(self, real_path):
        pass

    @exceptionally_logged
    @verbosely_logged
    def process_created(self, real_path):
        if real_path.endswith(os.path.sep):
            log_critical(
              u'GaminSourceTreeMonitor.process_created: ignoring %s' %
              real_path
            )
            return
        return self.add_cb(real_path)

    @exceptionally_logged
    @verbosely_logged
    def process_unexpected(self, real_path):
        log_critical(
          u'GaminSourceTreeMonitor.process_unexpected: %s' % real_path)

    @exceptionally_logged
    @verbosely_logged
    def event_callback(self, entry, kind, watch_path):
        real_path = os.path.join(watch_path, entry)
        return getattr(self, self.event_processors[kind])(real_path)

################################################################################

class DeferredGaminSourceTreeMonitor(
  DeferredSourceTreeMonitor, GaminSourceTreeMonitor):
    pass

==> pytagsfs_0.9.0.orig.tar.gz: pytagsfs-0.9.0/modules/pytagsfs/sourcetreemon/gamin_.py <==
# Copyright (c) 2008 Forest Bond.
# This file is part of the pytagsfs software package.
#
# pytagsfs is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# A copy of the license has been included in the COPYING file.

import os

from threading import RLock

from pytagsfs.util import nonatomic, attr_named
from pytagsfs.sourcetreemon import SourceTreeMonitor
from pytagsfs.sourcetreemon.deferred import DeferredSourceTreeMonitor
from pytagsfs.debug import log_critical
from pytagsfs.exceptions import MissingDependency

################################################################################

class GaminSourceTreeMonitor(SourceTreeMonitor):
    lock = None
    watch_monitor = None

    def __init__(self):
        try:
            import gamin
        except ImportError:
            raise MissingDependency('gamin')

        self.gamin = gamin

        self.event_processors = {
          gamin.GAMChanged: 'process_changed',
          gamin.GAMDeleted: 'process_deleted',
          gamin.GAMStartExecuting: 'process_ignored',
          gamin.GAMStopExecuting: 'process_ignored',
          gamin.GAMCreated: 'process_created',
          gamin.GAMMoved: 'process_deleted',
          gamin.GAMAcknowledge: 'process_ignored',
          gamin.GAMExists: 'process_unexpected',
          gamin.GAMEndExist: 'process_unexpected',
        }

        self.lock = RLock()

################################################################################

    # SourceTreeMonitor API

    def start(self, debug = False):
        self.watch_monitor = self.gamin.WatchMonitor()
        self.watch_monitor.no_exists()

    @nonatomic(attr_named('lock'))
    def stop(self):
        for real_path in self.watch_monitor.objects:
            self.watch_monitor.stop_watch(real_path)

    def supports_threads(self):
        return True

    def supports_writes(self):
        return True

    def can_handle_fork(self):
        return True

    def add_source_dir(self, real_path):
        self.watch_dir(real_path)

    def remove_source_dir(self, real_path):
        self.unwatch_dir(real_path)

    def add_source_file(self, real_path):
        pass

    def remove_source_file(self, real_path):
        pass

    def fileno(self):
        return self.watch_monitor.get_fd()

    @nonatomic(attr_named('lock'))
    def process_events(self):
        return self.watch_monitor.handle_events()

################################################################################

    @nonatomic(attr_named('lock'))
    def watch_dir(self, real_path):
        if real_path in self.watch_monitor.objects:
            # already watching
            return

        try:
            self.watch_monitor.watch_directory(
              real_path,
              self.event_callback,
              real_path,
            )
        except self.gamin.GaminException:
            pass

    @nonatomic(attr_named('lock'))
    def unwatch_dir(self, real_path):
        if real_path not in self.watch_monitor.objects:
            return

        try:
            self.watch_monitor.stop_watch(real_path)
        except self.gamin.GaminException:
            pass

    def process_changed(self, real_path):
        if real_path.endswith(os.path.sep):
            # FIXME: Is log_critical necessary here?
            log_critical(
              u'GaminSourceTreeMonitor.process_changed: ignoring %s',
              real_path,
            )
            return
        return self.update_cb(real_path)

    def process_deleted(self, real_path):
        if real_path.endswith(os.path.sep):
            # FIXME: Is log_critical necessary here?
            log_critical(
              u'GaminSourceTreeMonitor.process_deleted: ignoring %s',
              real_path,
            )
            return
        return self.remove_cb(real_path)

    def process_ignored(self, real_path):
        pass

    def process_created(self, real_path):
        if real_path.endswith(os.path.sep):
            # FIXME: Is log_critical necessary here?
            log_critical(
              u'GaminSourceTreeMonitor.process_created: ignoring %s',
              real_path,
            )
            return
        return self.add_cb(real_path)

    def process_unexpected(self, real_path):
        # FIXME: Is log_critical necessary here?
        log_critical(
          u'GaminSourceTreeMonitor.process_unexpected: %s', real_path)

    def event_callback(self, entry, kind, watch_path):
        real_path = os.path.join(watch_path, entry)
        return getattr(self, self.event_processors[kind])(real_path)

################################################################################

class DeferredGaminSourceTreeMonitor(
  DeferredSourceTreeMonitor, GaminSourceTreeMonitor):
    pass
